__package__    = "mtdb"
__version__    = "0.1"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/mtdb/"
__cvsversion__ = "$Revision: 1.11 $"
__date__       = "$Date: 2007/08/28 07:00:55 $"
__copyright__  = "Copyright (c) 2007 Aaron Straup Cope. Perl Artistic License."

import types

class machinetagstore :

    def __init__ (self, dbconn) :
        self.__dbconn = dbconn

    #

    def namespaces (self, pred_id=None, value_id=None, ctx_id=0) :

        args = []
        cond = []

        sql = "SELECT DISTINCT(namespace_id) FROM machinetagstore"

        if pred_id :
            cond.append("predicate_id=?")
            args.append(pred_id)

        if value_id :
            cond.append("value_id=?")
            args.append(value_id)

        if ctx_id :
            cond.append("context_id=?")
            args.append(ctx_id)
            
        if len(cond) == 0 :
            return self.__dbconn.execute(sql)

        where = " AND ".join(cond)
        sql += " WHERE %s" % where

        return self.__dbconn.execute(sql, args)
        
    #

    def predicates (self, ns_id=None, value_id=None, ctx_id=0) :

        args = []
        cond = []

        sql = "SELECT DISTINCT(predicate_id) FROM machinetagstore"

        if ns_id :
            cond.append("namespace_id=?")
            args.append(ns_id)

        if value_id :
            cond.append("value_id=?")
            args.append(value_id)

        if ctx_id :
            cond.append("context_id=?")
            args.append(ctx_id)
        
        if len(cond) == 0 :
            return self.__dbconn.execute(sql)

        where = " AND ".join(cond)
        sql += " WHERE %s" % where

        return self.__dbconn.execute(sql, args)

    #

    def values (self, ns_id=None, pred_id=None, ctx_id=0) :

        args = []
        cond = []

        sql = "SELECT DISTINCT(value_id) FROM machinetagstore"

        if ns_id :
            cond.append("namespace_id=?")
            args.append(ns_id)

        if pred_id :
            cond.append("predicate_id=?")
            args.append(pred_id)

        if ctx_id :
            cond.append("context_id=?")
            args.append(ctx_id)

        if len(cond) == 0 :
            return self.__dbconn.execute(sql)

        where = " AND ".join(cond)
        sql += " WHERE %s" % where

        return self.__dbconn.execute(sql, args)

    #

    def store (self, ns_id, pred_id, value_id, doc_id, ctx_id, value) :

        args = (ns_id, pred_id, value_id, doc_id, ctx_id, value)

        val_type = type(value)
        val_col = "string_value"
        
        if val_type == types.IntType or val_type == types.FloatType :
            val_col = "float_value"
        
        sql = "INSERT INTO machinetagstore (namespace_id, predicate_id, value_id, document_id, context_id, %s) VALUES (?,?,?,?,?,?)" % val_col
        
        self.__dbconn.execute(sql, args)
        self.__dbconn.commit()

    #
    
    def remove_document (self, document_id, ctx_id=0) :

        args = [document_id]
        sql = "DELETE FROM machinetagstore WHERE document_id=?"

        if ctx_id :
            sql += " AND context_id=?"
            args.append(ctx_id)

        args = tuple(args)
        
        self.__dbconn.execute(sql, args)
        self.__dbconn.commit()
        
    #
    
    def remove_tag (self, document_id, ns_id, pred_id, value_id, ctx_id=0) :

        cond = ["document_id=?"]
        args = [document_id]

        if ns_id :
            cond.append("namespace_id=?")
            args.append(ns_id)

        if pred_id :
            cond.append("predicate_id=?")
            args.append(pred_id)

        if value_id :
            cond.append("value_id=?")
            args.append(value_id)

        if ctx_id :
            cond.append("context_id=?")
            args.append(ctx_id)

        args = tuple(args)
        
        where = " AND ".join(cond)
        sql = "DELETE FROM machinetagstore WHERE %s" % where
        
        return self.__dbconn.execute(sql, args)
                    
    #
    
    def retrieve_by_document (self, doc_id, ctx_id=0) :

        args = [doc_id]
        sql = "SELECT * FROM machinetagstore WHERE document_id=?"

        if ctx_id :
            sql += " AND context_id=?"
            args.append(ctx_id)

        args = tuple(args)
        return self.__dbconn.execute(sql, args)

    #

    def retrieve_by_machinetag(self, ns_id=None, pred_id=None, value_id=None, ctx_id=0) :

        sql, args = self.__sql_retrieve_by_machinetag(ns_id, pred_id, value_id, ctx_id)
        args = tuple(args)

        return self.__dbconn.execute(sql, args)

    #
    
    def retrieve_documents_by_machinetag(self, ns_id=None, pred_id=None, value_id=None, ctx_id=0) :

        sql, args = self.__sql_retrieve_by_machinetag(ns_id, pred_id, value_id, ctx_id)

        sql += " GROUP BY document_id"
        args = tuple(args)

        return self.__dbconn.execute(sql, args)

    #

    def retrieve_by_machinetag_range(self, value_range, ns_id=None, pred_id=None, ctx_id=0) :

        sql, args = self.__sql_retrieve_by_machinetag_range(value_range, ns_id, pred_id, ctx_id)
        args = tuple(args)
        
        return self.__dbconn.execute(sql, args)

    #
    
    def retrieve_documents_by_machinetag_range(self, value_range, ns_id=None, pred_id=None, ctx_id=0) :

        sql, args = self.__sql_retrieve_by_machinetag_range(value_range, ns_id, pred_id, ctx_id)

        sql += " GROUP BY document_id"
        args = tuple(args)
        
        return self.__dbconn.execute(sql, args)

    #
    
    def retrieve_by_machinetag_like(self, like, ns_id=None, pred_id=None, ctx_id=0) :

        sql, args = self.__sql_retrieve_by_machinetag_like(like, ns_id, pred_id, ctx_id)
        args = tuple(args)
        
        return self.__dbconn.execute(sql, args)

    #
    
    def retrieve_documents_by_machinetag_like(self, like, ns_id=None, pred_id=None, ctx_id=0) :

        sql, args = self.__sql_retrieve_by_machinetag_like(like, ns_id, pred_id, ctx_id)

        sql += " GROUP BY document_id"
        args = tuple(args)
        
        return self.__dbconn.execute(sql, args)

    #

    def __sql_retrieve_by_machinetag_range (self, value_range, ns_id=None, pred_id=None, ctx_id=0) :
        
        cond = []
        args = []

        cond.append("float_value BETWEEN ? and ?")
        args.append(value_range[0])
        args.append(value_range[1])        

        if ns_id :
            cond.append("namespace_id=?")
            args.append(ns_id)
            
        if pred_id :
            cond.append("predicate_id=?")
            args.append(pred_id)
            
        if ctx_id :
            cond.append("context_id=?")
            args.append(ctx_id)
                
        where = " AND ".join(cond)        
        sql = "SELECT * FROM machinetagstore WHERE %s" % where
        
        return [sql, args]

    #

    def __sql_retrieve_by_machinetag_like (self, like, ns_id=None, pred_id=None, ctx_id=0) :
        
        cond = []
        args = []

        cond.append("string_value LIKE ?")
        args.append(like)

        if ns_id :
            cond.append("namespace_id=?")
            args.append(ns_id)
            
        if pred_id :
            cond.append("predicate_id=?")
            args.append(pred_id)
            
        if ctx_id :
            cond.append("context_id=?")
            args.append(ctx_id)
                
        where = " AND ".join(cond)        
        sql = "SELECT * FROM machinetagstore WHERE %s" % where
        
        return [sql, args]
    
    #
    
    def __sql_retrieve_by_machinetag (self, ns_id=None, pred_id=None, value_id=None, ctx_id=0) :

        cond = []
        args = []
        
        if ns_id :
            cond.append("namespace_id=?")
            args.append(ns_id)
            
        if pred_id :
            cond.append("predicate_id=?")
            args.append(pred_id)
            
        if value_id :
            cond.append("value_id=?")
            args.append(value_id)
            
        if ctx_id :
            cond.append("context_id=?")
            args.append(ctx_id)
            
        where = " AND ".join(cond)        
        sql = "SELECT * FROM machinetagstore WHERE %s" % where

        return [sql, args]
